@@ -2,20 +2,20 @@
 
 ##' Inserts data into a PostgreSQL table.
 ##'
-##' This function takes a take an R \code{sp} object (Spatial* or
-##' Spatial*DataFrame), or a regular data frame, and performs the
+##' This function takes a take an R \code{sp} object (\code{Spatial*} or
+##' \code{Spatial*DataFrame}), or a regular \code{data.frame}, and performs the
 ##' database insert (and table creation, when the table doesn't exist)
 ##' on the database.
 ##'
 ##' If \code{new.id} is specified, a new sequential integer field is
 ##' added to the data frame for insert. For \code{Spatial*}-only
-##' objects (no data frame), a new.id is created by default with name
-##' "gid".
+##' objects (no data frame), a new ID column is created by default with name
+##' \code{"gid"}.
 ##'
 ##' If the R package \code{wkb} is installed, this function will use
-##' \code{writeWKB} for certain datasets (non-Multi types,
+##' \code{\link[wkb]{writeWKB}} for certain datasets (non-Multi types,
 ##' non-Linestring), which is faster for large datasets.  In all other
-##' cases the \code{rgeos} function \code{writeWKT} is used.
+##' cases the \code{rgeos} function \code{\link[rgeos]{writeWKT}} is used.
 ##'
 ##' In the event of function or database error, the database uses
 ##' ROLLBACK to revert to the previous state. 
@@ -27,7 +27,7 @@
 ##' or re-used as the \code{data.obj} in \code{pgInsert}; 
 ##' (e.g., when data preparation is slow, and the exact same data 
 ##' needs to be inserted into tables in two separate
-##' tables or databases).If \code{return.pgi = FALSE}
+##' tables or databases). If \code{return.pgi = FALSE}
 ##' (default), the function will return \code{TRUE} for successful insert and
 ##' \code{FALSE} for failed inserts.
 ##' 
@@ -40,8 +40,9 @@
 ##' 
 ##' 
 ##' @param conn A connection object to a PostgreSQL database
-##' @param name Character, schema and table of the PostgreSQL table to
-##'     insert into. If not already existing, the table will be
+##' @param name A character string specifying a PostgreSQL schema and
+##'     table name (e.g., \code{name = c("schema","table")}). 
+##'     If not already existing, the table will be
 ##'     created. If the table already exists, the function will check
 ##'     if all R data frame columns match database columns, and if so,
 ##'     do the insert. If not, the insert will be aborted. The
@@ -49,10 +50,15 @@
 ##'     partial matches of data frame and database column names, and
 ##'     \code{overwrite} allows for overwriting the existing database
 ##'     table.
-##' @param data.obj A Spatial* or Spatial*DataFrame, or data frame
-##' @param geom character string. For Spatial* datasets, the name of
+##' @param data.obj A \code{Spatial*} or \code{Spatial*DataFrame}, or \code{data.frame}
+##' @param geom character string. For \code{Spatial*} datasets, the name of
 ##'     geometry column in the database table.  (existing or to be
 ##'     created; defaults to \code{"geom"}).
+##' @param df.mode Logical; Whether to write the (Spatial) data frame in data frame mode 
+##'     (preserving data frame column attributes and row.names).
+##'     A new table must be created with this mode (or overwrite set to TRUE),
+##'     and the \code{row.names}, \code{alter.names}, and \code{new.id} arguments will
+##'     be ignored (see \code{\link[rpostgis]{dbWriteDataFrame}} for more information).
 ##' @param partial.match Logical; allow insert on partial column
 ##'     matches between data frame and database table. If \code{TRUE},
 ##'     columns in R data frame will be compared with the existing
@@ -67,18 +73,20 @@
 ##'     and defaults to the name \code{"gid"}). If \code{partial.match
 ##'     = TRUE} and the column does not exist in the databse table,
 ##'     it will be discarded.
+##' @param row.names Whether to add the data frame row names to the 
+##'     database table. Column name will be '.R_rownames'.
 ##' @param upsert.using Character, name of the column(s) in the database table 
 ##'     or constraint name used to identify already-existing rows in the table, which will
 ##'     be updated rather than inserted. The column(s) must have a unique constraint
 ##'     already created in the database table (e.g., a primary key). 
 ##'     Requires PostgreSQL 9.5+.
 ##' @param alter.names Logical, whether to make database column names
-##'     DB-compliant (remove special characters). Default is
-##'     \code{TRUE}.  (This should to be set to \code{FALSE} to match
-##'     to non-standard names in an existing database table.)
+##'     DB-compliant (remove special characters/capitalization). Default is
+##'     \code{FALSE}.  (This must be set to \code{FALSE} to match
+##'     with non-standard names in an existing database table.)
 ##' @param encoding Character vector of length 2, containing the
 ##'     from/to encodings for the data (as in the function
-##'     \code{iconv}). For example, if the dataset contain certain
+##'     \code{\link[base]{iconv}}). For example, if the dataset contain certain
 ##'     latin characters (e.g., accent marks), and the database is in
 ##'     UTF-8, use \code{encoding = c("latin1", "UTF-8")}. Left
 ##'     \code{NULL}, no conversion will be done.
@@ -109,9 +117,23 @@
 ##'     partial.match = TRUE)
 ##' }
 
-pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE, 
-    overwrite = FALSE, new.id = NULL, upsert.using = NULL, alter.names = TRUE, encoding = NULL, 
-    return.pgi = FALSE) {
+pgInsert <- function(conn, name, data.obj, geom = "geom", df.mode = FALSE, partial.match = FALSE, 
+    overwrite = FALSE, new.id = NULL, row.names = FALSE, upsert.using = NULL,
+    alter.names = FALSE, encoding = NULL, return.pgi = FALSE) {
+  
+    if (df.mode) {
+      if (!dbExistsTable(conn,name) | overwrite) {
+        # set necessary argument values
+        partial.match <- FALSE
+        new.id <- ".db_pkid"
+        row.names <- TRUE
+        upsert.using <- NULL
+        alter.names <- FALSE
+      } else if (!overwrite & dbExistsTable(conn,name)) {
+        stop("df.mode = TRUE only allowed for new tables or with overwrite = TRUE.")
+      }
+    }
+  
     dbConnCheck(conn)
     ## Check if PostGIS installed
     if (!suppressMessages(pgPostGIS(conn))) {
@@ -147,7 +169,6 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
         force.match <- name
         create.table <- NULL
     }
-    dbSendQuery(conn, "BEGIN TRANSACTION;")
     geo.classes <- c("SpatialPoints", "SpatialPointsDataFrame", 
         "SpatialLines", "SpatialLinesDataFrame", "SpatialPolygons", 
         "SpatialPolygonsDataFrame")
@@ -156,21 +177,23 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
         try(suppressMessages(pgSRID(conn, data.obj@proj4string, 
             create.srid = TRUE, new.srid = NULL)), silent = TRUE)
         try(pgi <- pgInsertizeGeom(data.obj, geom, create.table, 
-            force.match, conn, new.id, alter.names, partial.match))
+            force.match, conn, new.id, row.names, alter.names, partial.match, df.mode))
     } else if (cls == "data.frame") {
         try(pgi <- pgInsertize(data.obj, create.table, force.match, 
-            conn, new.id, alter.names, partial.match))
+            conn, new.id, row.names, alter.names, partial.match, df.mode))
     } else if (cls == "pgi") {
         pgi <- data.obj
         message("Using previously create pgi object. All arguments except for \"conn\", \"overwrite\", and \"encoding\" will be ignored.")
     } else {
-        dbSendQuery(conn, "ROLLBACK;")
+        #dbExecute(conn, "ROLLBACK;")
         stop("Input data object not of correct class - must be a Spatial*, Spatial*DataFrame, or data frame.")
     }
     if (is.null(pgi)) {
-        dbSendQuery(conn, "ROLLBACK;")
+        #dbExecute(conn, "ROLLBACK;")
         stop("Table preparation failed. No changes made to database.")
     }
+    
+    dbExecute(conn, "BEGIN TRANSACTION;")
     ## Change encoding if specified
     if (!is.null(encoding)) {
         pgi$insert.data <- iconv(pgi$insert.data, encoding[1], 
@@ -182,7 +205,7 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
             over.t <- dbDrop(conn, name = name, type = "table", 
                 ifexists = TRUE)
             if (!over.t) {
-                dbSendQuery(conn, "ROLLBACK;")
+                dbExecute(conn, "ROLLBACK;")
                 message("Could not drop existing table. No changes made to database.")
                 if (return.pgi) {
                   return(pgi)
@@ -192,9 +215,9 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
             }
         }
         quet <- NULL
-        try(quet <- dbSendQuery(conn, pgi$db.new.table))
+        try(quet <- dbExecute(conn, pgi$db.new.table))
         if (is.null(quet)) {
-            dbSendQuery(conn, "ROLLBACK;")
+            dbExecute(conn, "ROLLBACK;")
             message("Table creation failed. No changes made to database.")
             if (return.pgi) {
                 return(pgi)
@@ -214,7 +237,7 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
     values <- pgi$insert.data
     db.cols <- dbTableInfo(conn, name = name)$column_name
     if (is.null(db.cols)) {
-        dbSendQuery(conn, "ROLLBACK;")
+        dbExecute(conn, "ROLLBACK;")
         message(paste0("Database table ", paste(name, collapse = "."), 
             " not found; No changes made to database."))
         if (return.pgi) {
@@ -226,7 +249,7 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
     test <- match(cols, db.cols)
     unmatched <- cols[is.na(test)]
     if (length(unmatched) > 0) {
-        dbSendQuery(conn, "ROLLBACK;")
+        dbExecute(conn, "ROLLBACK;")
         message(paste0("The column(s) (", paste(unmatched, collapse = ","), 
             ") are not in the database table. No changes made to database."))
         if (return.pgi) {
@@ -257,9 +280,10 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
     ## Send insert query
     temp.query<-paste0("INSERT INTO ", nameque[1], 
         ".", nameque[2], cols2, " VALUES ", values, up.query,";")
-    try(quei <- dbSendQuery(conn, temp.query))
+    try(quei <- dbExecute(conn, temp.query))
     if (!is.null(quei)) {
-        dbSendQuery(conn, "COMMIT;")
+        if (df.mode) {dbAddKey(conn, name, colname = ".db_pkid", type = "primary")}
+        dbExecute(conn, "COMMIT;")
         message(paste0("Data inserted into table ",nameque[1],".",nameque[2]))
         ## Return TRUE
         if (return.pgi) {
@@ -268,7 +292,7 @@ pgInsert <- function(conn, name, data.obj, geom = "geom", partial.match = FALSE,
             return(TRUE)
         }
     } else {
-        dbSendQuery(conn, "ROLLBACK;")
+        dbExecute(conn, "ROLLBACK;")
         message("Insert failed. No changes made to database.")
         if (return.pgi) {
             return(pgi)
